@@ -21,7 +21,7 @@ import logging
 
 import six
 
-import saml2.version
+from saml2.version import version as __version__
 from saml2.validate import valid_instance
 
 try:
@@ -40,11 +40,7 @@ except ImportError:
 import defusedxml.ElementTree
 
 
-__version__ = str(saml2.version.version)
-
-
-root_logger = logging.getLogger(__name__)
-root_logger.level = logging.NOTSET
+logger = logging.getLogger(__name__)
 
 NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:assertion'
 # TEMPLATE = '{urn:oasis:names:tc:SAML:2.0:assertion}%s'
