@@ -470,7 +470,7 @@ class SAML2Plugin(object):
 
         is_request = "SAMLRequest" in query
         is_response = "SAMLResponse" in query
-        has_content_length = "CONTENT_LENGTH" in environ or environ["CONTENT_LENGTH"]
+        has_content_length = environ.get("CONTENT_LENGTH")
 
         if not has_content_length and not is_request and not is_response:
             logger.debug("[identify] get or empty post")
@@ -670,7 +670,7 @@ def make_plugin(
     discovery="",
     idp_query_param="",
 ):
-    if saml_conf is "":
+    if saml_conf == "":
         raise ValueError("must include saml_conf in configuration")
 
     if remember_name is None:
