@@ -3,6 +3,8 @@
 #
 # Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
 #
+import base64
+
 from saml2.validate import valid_ipv4, MustValueError
 from saml2.validate import valid_ipv6
 from saml2.validate import ShouldValueError
@@ -11,8 +13,9 @@ from saml2.validate import valid_domain_name
 import saml2
 from saml2 import SamlBase
 
-import xmldsig as ds
-import xmlenc as xenc
+import six
+from saml2 import xmldsig as ds
+from saml2 import xmlenc as xenc
 
 NAMESPACE = 'urn:oasis:names:tc:SAML:2.0:assertion'
 
@@ -22,20 +25,24 @@ XS_NAMESPACE = 'http://www.w3.org/2001/XMLSchema'
 XSI_TYPE = '{%s}type' % XSI_NAMESPACE
 XSI_NIL = '{%s}nil' % XSI_NAMESPACE
 
+NAMEID_FORMAT_UNSPECIFIED = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified")
 NAMEID_FORMAT_EMAILADDRESS = (
     "urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress")
-NAMEID_FORMAT_UNSPECIFIED1 = (
-    "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified")
-NAMEID_FORMAT_UNSPECIFIED = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:unspecified")
-NAMEID_FORMAT_ENCRYPTED = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted")
+NAMEID_FORMAT_X509SUBJECTNAME = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName")
+NAMEID_FORMAT_WINDOWSDOMAINQUALIFIEDNAME = (
+    "urn:oasis:names:tc:SAML:1.1:nameid-format:WindowsDomainQualifiedName")
+NAMEID_FORMAT_KERBEROS = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:kerberos")
+NAMEID_FORMAT_ENTITY = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:entity")
 NAMEID_FORMAT_PERSISTENT = (
     "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent")
 NAMEID_FORMAT_TRANSIENT = (
     "urn:oasis:names:tc:SAML:2.0:nameid-format:transient")
-NAMEID_FORMAT_ENTITY = (
-    "urn:oasis:names:tc:SAML:2.0:nameid-format:entity")
+NAMEID_FORMAT_ENCRYPTED = (
+    "urn:oasis:names:tc:SAML:2.0:nameid-format:encrypted")
 NAMEID_FORMATS_SAML2 = (
     ('NAMEID_FORMAT_EMAILADDRESS', NAMEID_FORMAT_EMAILADDRESS),
     ('NAMEID_FORMAT_ENCRYPTED', NAMEID_FORMAT_ENCRYPTED),
@@ -76,50 +83,9 @@ SCM_HOLDER_OF_KEY = "urn:oasis:names:tc:SAML:2.0:cm:holder-of-key"
 SCM_SENDER_VOUCHES = "urn:oasis:names:tc:SAML:2.0:cm:sender-vouches"
 SCM_BEARER = "urn:oasis:names:tc:SAML:2.0:cm:bearer"
 
-# -----------------------------------------------------------------------------
-XSD = "xs:"
+XSD = "xs"
 NS_SOAP_ENC = "http://schemas.xmlsoap.org/soap/encoding/"
 
-# -----------------------------------------------------------------------------
-
-
-def _decode_attribute_value(typ, text):
-    if typ == XSD + "string":
-        return text or ""
-    if typ == XSD + "integer" or typ == XSD + "int":
-        return str(int(text))
-    if typ == XSD + "float" or typ == XSD + "double":
-        return str(float(text))
-    if typ == XSD + "boolean":
-        return "%s" % (text == "true" or text == "True")
-    if typ == XSD + "base64Binary":
-        import base64
-
-        return base64.decodestring(text)
-    raise ValueError("type %s not supported" % type)
-
-
-def _verify_value_type(typ, val):
-    #print "verify value type: %s, %s" % (typ, val)
-    if typ == XSD + "string":
-        try:
-            return str(val)
-        except UnicodeEncodeError:
-            return unicode(val)
-    if typ == XSD + "integer" or typ == XSD + "int":
-        return int(val)
-    if typ == XSD + "float" or typ == XSD + "double":
-        return float(val)
-    if typ == XSD + "boolean":
-        if val.lower() == "true" or val.lower() == "false":
-            pass
-        else:
-            raise ValueError("Faulty boolean value")
-    if typ == XSD + "base64Binary":
-        import base64
-
-        return base64.decodestring(val)
-
 
 class AttributeValueBase(SamlBase):
     def __init__(self,
@@ -135,10 +101,12 @@ class AttributeValueBase(SamlBase):
         if self._extatt:
             self.extension_attributes = self._extatt
 
-        if not text:
-            self.extension_attributes = {XSI_NIL: 'true'}
-        else:
+        if text:
             self.set_text(text)
+        elif not extension_elements:
+            self.extension_attributes = {XSI_NIL: 'true'}
+        elif XSI_TYPE in self.extension_attributes:
+            del self.extension_attributes[XSI_TYPE]
 
     def __setattr__(self, key, value):
         if key == "text":
@@ -157,7 +125,7 @@ class AttributeValueBase(SamlBase):
     def set_type(self, typ):
         try:
             del self.extension_attributes[XSI_NIL]
-        except KeyError:
+        except (AttributeError, KeyError):
             pass
 
         try:
@@ -165,6 +133,12 @@ class AttributeValueBase(SamlBase):
         except AttributeError:
             self._extatt[XSI_TYPE] = typ
 
+        if typ.startswith('xs:'):
+            try:
+                self.extension_attributes['xmlns:xs'] = XS_NAMESPACE
+            except AttributeError:
+                self._extatt['xmlns:xs'] = XS_NAMESPACE
+
     def get_type(self):
         try:
             return self.extension_attributes[XSI_TYPE]
@@ -184,86 +158,142 @@ class AttributeValueBase(SamlBase):
         except KeyError:
             pass
 
-    def set_text(self, val, base64encode=False):
-        typ = self.get_type()
-        if base64encode:
-            import base64
+    def set_text(self, value, base64encode=False):
+        def _wrong_type_value(xsd, value):
+            msg = 'Type and value do not match: {xsd}:{type}:{value}'
+            msg = msg.format(xsd=xsd, type=type(value), value=value)
+            raise ValueError(msg)
 
-            val = base64.encodestring(val)
-            self.set_type("xs:base64Binary")
-        else:
-            if isinstance(val, basestring):
-                if not typ:
-                    self.set_type("xs:string")
-                else:
-                    try:
-                        assert typ == "xs:string"
-                    except AssertionError:
-                        if typ == "xs:int":
-                            _ = int(val)
-                        elif typ == "xs:boolean":
-                            if val.lower() not in ["true", "false"]:
-                                raise ValueError("Not a boolean")
-                        elif typ == "xs:float":
-                            _ = float(val)
-                        elif typ == "xs:base64Binary":
-                            pass
-                        else:
-                            ValueError("Type and value doesn't match")
-            elif isinstance(val, bool):
-                if val:
-                    val = "true"
-                else:
-                    val = "false"
-                if not typ:
-                    self.set_type("xs:boolean")
-                else:
-                    assert typ == "xs:boolean"
-            elif isinstance(val, int):
-                val = str(val)
-                if not typ:
-                    self.set_type("xs:integer")
-                else:
-                    assert typ == "xs:integer"
-            elif isinstance(val, float):
-                val = str(val)
-                if not typ:
-                    self.set_type("xs:float")
-                else:
-                    assert typ == "xs:float"
-            elif not val:
-                try:
-                    self.extension_attributes[XSI_TYPE] = typ
-                except AttributeError:
-                    self._extatt[XSI_TYPE] = typ
-                val = ""
-            else:
-                if typ == "xs:anyType":
-                    pass
-                else:
-                    raise ValueError
+        # only work with six.string_types
+        if isinstance(value, six.binary_type):
+            value = value.decode('utf-8')
 
-        SamlBase.__setattr__(self, "text", val)
+        type_to_xsd = {
+            str:        'string',
+            int:        'integer',
+            float:      'float',
+            bool:       'boolean',
+            type(None): '',
+        }
+
+        # entries of xsd-types each declaring:
+        # - a corresponding python type
+        # - a function to turn a string into that type
+        # - a function to turn that type into a text-value
+        xsd_types_props = {
+            'string': {
+                'type': str,
+                'to_type': str,
+                'to_text': str,
+            },
+            'integer': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'short': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'int': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'long': {
+                'type': int,
+                'to_type': int,
+                'to_text': str,
+            },
+            'float': {
+                'type': float,
+                'to_type': float,
+                'to_text': str,
+            },
+            'double': {
+                'type': float,
+                'to_type': float,
+                'to_text': str,
+            },
+            'boolean': {
+                'type': bool,
+                'to_type': lambda x: {
+                    'true': True,
+                    'false': False,
+                }[str(x).lower()],
+                'to_text': lambda x: str(x).lower(),
+            },
+            'base64Binary': {
+                'type': str,
+                'to_type': str,
+                'to_text': (
+                    lambda x: base64.encodebytes(x.encode()) if base64encode else x
+                ),
+            },
+            'anyType': {
+                'type': type(value),
+                'to_type': lambda x: x,
+                'to_text': lambda x: x,
+            },
+            '': {
+                'type': type(None),
+                'to_type': lambda x: None,
+                'to_text': lambda x: '',
+            },
+        }
+
+        xsd_string = (
+            'base64Binary' if base64encode
+            else self.get_type()
+            or type_to_xsd.get(type(value)))
+
+        xsd_ns, xsd_type = (
+            ['', type(None)] if xsd_string is None
+            else ['', ''] if xsd_string == ''
+            else [
+                XSD if xsd_string in xsd_types_props else '',
+                xsd_string
+            ] if ':' not in xsd_string
+            else xsd_string.split(':', 1))
+
+        xsd_type_props = xsd_types_props.get(xsd_type, {})
+        valid_type = xsd_type_props.get('type', type(None))
+        to_type = xsd_type_props.get('to_type', str)
+        to_text = xsd_type_props.get('to_text', str)
+
+        # cast to correct type before type-checking
+        if type(value) is str and valid_type is not str:
+            try:
+                value = to_type(value)
+            except (TypeError, ValueError, KeyError) as e:
+                # the cast failed
+                _wrong_type_value(xsd=xsd_type, value=value)
+
+        if type(value) is not valid_type:
+            _wrong_type_value(xsd=xsd_type, value=value)
+
+        text = to_text(value)
+        self.set_type(
+            '{ns}:{type}'.format(ns=xsd_ns, type=xsd_type) if xsd_ns
+            else xsd_type if xsd_type
+            else '')
+        SamlBase.__setattr__(self, 'text', text)
         return self
 
     def harvest_element_tree(self, tree):
         # Fill in the instance members from the contents of the XML tree.
         for child in tree:
             self._convert_element_tree_to_member(child)
-        for attribute, value in tree.attrib.iteritems():
+        for attribute, value in iter(tree.attrib.items()):
             self._convert_element_attribute_to_member(attribute, value)
         if tree.text:
-            #print "set_text:", tree.text
+            #print("set_text:", tree.text)
             # clear type
             #self.clear_type()
             self.set_text(tree.text)
             if XSI_NIL in self.extension_attributes:
                 del self.extension_attributes[XSI_NIL]
-            try:
-                typ = self.extension_attributes[XSI_TYPE]
-                _verify_value_type(typ, getattr(self, "text"))
-            except KeyError:
-                pass
 
 
 class BaseIDAbstractType_(SamlBase):
@@ -438,7 +468,7 @@ class SubjectConfirmationDataType_(SamlBase):
     c_attributes = SamlBase.c_attributes.copy()
     c_child_order = SamlBase.c_child_order[:]
     c_cardinality = SamlBase.c_cardinality.copy()
-    c_attributes['NotBefore'] = ('not_before', 'AsTime', False)
+    c_attributes['NotBefore'] = ('not_before', 'dateTime', False)
     c_attributes['NotOnOrAfter'] = ('not_on_or_after', 'dateTime', False)
     c_attributes['Recipient'] = ('recipient', 'anyURI', False)
     c_attributes['InResponseTo'] = ('in_response_to', 'NCName', False)
@@ -796,7 +826,16 @@ def base_id_from_string(xml_string):
 
 
 class NameID(NameIDType_):
-    """The urn:oasis:names:tc:SAML:2.0:assertion:NameID element """
+    """The urn:oasis:names:tc:SAML:2.0:assertion:NameID element
+
+    From the Oasis SAML2 Technical Overview:
+
+    "The <NameID> element within a <Subject> offers the ability to provide name
+     identifiers in a number of different formats. SAML's predefined formats
+     include: Email address, X.509 subject name, Windows domain qualified name,
+     Kerberos principal name, Entity identifier, Persistent identifier,
+     Transient identifier."
+     """
 
     c_tag = 'NameID'
     c_namespace = NAMESPACE
